import os

import requests


def check_url_accessible(url: str) -> bool:
    try:
        response = requests.head(url, timeout=3)
        return response.ok  # True si status_code ∈ [200, 399]
    except requests.RequestException:
        return False

def check_gitlab_url(gitlab_url: str) -> bool:
    return check_url_accessible(gitlab_url)

def check_gitlab_token(gitlab_url: str, gitlab_token: str) -> bool:
    headers = {
        'Private-Token': gitlab_token
    }

    try:
        response = requests.get(gitlab_url + 'api/v4/user', headers=headers, timeout=3)
        return response.ok  # True si status_code ∈ [200, 399]
    except requests.RequestException:
        return False

def check_gitlab_group(gitlab_url: str, gitlab_token: str, gitlab_group: str) -> bool:
    headers = {
        'Private-Token': gitlab_token
    }

    try:
        response = requests.get(gitlab_url + f'api/v4/groups/{gitlab_group}', headers=headers, timeout=3)
        return response.ok  # True si status_code ∈ [200, 399]
    except requests.RequestException:
        return False

def check_gitlab_username(gitlab_url: str, gitlab_token: str, gitlab_username: str) -> bool:
    headers = {
        'Private-Token': gitlab_token
    }

    try:
        response = requests.get(gitlab_url + f'api/v4/users/{gitlab_username}', headers=headers, timeout=3)
        return response.ok  # True si status_code ∈ [200, 399]
    except requests.RequestException:
        return False

def check_github_repo(github_repo: str) -> bool:
    if not github_repo:
        return False

    parts = github_repo.split('/')
    if len(parts) != 2:
        return False

    owner, repo_name = parts
    if not owner or not repo_name:
        return False

    url = f"https://api.github.com/repos/{owner}/{repo_name}"
    return check_url_accessible(url)

def check_config_variables(key: str, value: str) -> bool:
    if key == "GITHUB_REPO":
        return check_github_repo(value)
    elif key == "GITLAB_REPO":
        return True
    elif key == "GITLAB_URL":
        return check_gitlab_url(value)
    elif key == "GITLAB_TOKEN":
        return True
    elif key == "GITLAB_GROUP":
        return True
    elif key == "GITLAB_USERNAME":
        return True
    else:
        return False



import os

from hub_to_lab.check import check_config_variables
from hub_to_lab.config import update_env_variable
from hub_to_lab.github import clone_github_repo
from hub_to_lab.gitlab import get_or_create_gitlab_project
from hub_to_lab.gitops import push_to_gitlab
from rich.progress import Progress, SpinnerColumn, BarColumn, TextColumn, TimeElapsedColumn
import typer

app = typer.Typer()

@app.command()
def main():

    config = {
        "GITHUB_REPO": "Name of the GitHub repository (ex: torvalds/linux)",
        "GITLAB_REPO": "Name of the new project on Gitlab",
        "GITLAB_URL": "GitLab URL",
        "GITLAB_TOKEN": "GitLab TOKEN (PAT)",
        "GITLAB_USERNAME": "GitLab Username",
        "GITLAB_GROUP": "GitLab Group name (use '-' for personal namespace or leave if precedent group was your personal namespace)",
    }


    github_repo, new_repo_name, gitlab_url, gitlab_token, gitlab_user, gitlab_group = setup_config_variables(config)

    github_clone_url = f"https://github.com/{github_repo}.git"

    print("Create the Gitlab project...")

    project = get_or_create_gitlab_project(
        gitlab_url=gitlab_url,
        token=gitlab_token,
        username=gitlab_user,
        group=gitlab_group,
        name=new_repo_name
    )

    remote_url = project["http_url_to_repo"]

    remote_url = remote_url.replace("https://", f"https://oauth2:{gitlab_token}@")

    with Progress(
        SpinnerColumn(),
        BarColumn(),
        TextColumn("[progress.description]{task.description}"),
        TimeElapsedColumn()
    ) as progress:
        repo = clone_github_repo(github_clone_url, progress)

        push_to_gitlab(repo, remote_url, progress)

    print("✅ Fini.")


def setup_config_variables(config: dict):
    config_variables = []
    for key, prompt in config.items():
        while True:
            variable = typer.prompt(prompt, default=os.getenv(key))
            if check_config_variables(key, variable):
                if key not in ["GITHUB_REPO", "GITLAB_REPO"]:
                    if key == "GITLAB_GROUP" and variable == "-":
                        variable = ''
                    update_env_variable(key, variable)
                config_variables.append(variable)
                break
            else:
                typer.echo(f"Error: Invalid value for {key}. Please try again.")
    return config_variables


if __name__ == "__main__":
    main()



import requests


def get_or_create_gitlab_project(gitlab_url, token, username, group, name):
    api_url = f"{gitlab_url}/api/v4"
    headers = {"PRIVATE-TOKEN": token}


    if group != '':
        if not check_groups(gitlab_url, token, group):
            print(f"Group '{group}' does not exist on Gitlab. Please create it first.")
            exit()

    try:
        search_url = f"{api_url}/groups/{group}/projects?search={name}" if group != '' else f"{api_url}/projects?search={name}"
        r = requests.get(search_url, headers=headers)
        r.raise_for_status()

        if r.json() != []:
            print("Project already exist on Gitlab:", r.json()[0]["web_url"])
            return r.json()[0]

        url = f"{api_url}/projects"
        url = url.replace("https://", f"https://{username}:{token}@")
        data = {"name": name}
        if group:
            group_url = f"{api_url}/groups/{group}"
            group_resp = requests.get(group_url, headers=headers)
            group_resp.raise_for_status()
            data["namespace_id"] = group_resp.json()["id"]

        response = requests.post(url, headers=headers, json=data)
        response.raise_for_status()
        print("Gitlab project created :", response.json()["web_url"])
        return response.json()
    except requests.exceptions.RequestException as e:
        print(f"Error during the communication to Gitlab: {e}")
        print("Verify your connection, URL, tokens and permissions.")
        raise

def check_groups(gitlab_url, token, group):
    api_url = f"{gitlab_url}/api/v4"
    headers = {"PRIVATE-TOKEN": token}

    url = f"{api_url}/groups/{group}"
    response = requests.get(url, headers=headers)
    if response.status_code != 200:
        return False
    return True



def push_to_gitlab(repo, remote_url, progress):
    task = progress.add_task("[cyan]Push to GitLab...", total=None)

    if "origin" in repo.remotes:
        repo.delete_remote("origin")
    origin = repo.create_remote("origin", remote_url)
    origin.push(all=True)
    origin.push(tags=True)

    progress.update(task, completed=1)
    progress.remove_task(task)



import tempfile
from git import Repo


def clone_github_repo(github_url, progress):
    tmpdir = tempfile.mkdtemp()
    task = progress.add_task("[green]Clone GitHub...", total=None)

    repo = Repo.clone_from(github_url, tmpdir)
    progress.update(task, completed=1)
    progress.remove_task(task)
    return repo


import os

from dotenv import load_dotenv, set_key
from platformdirs import user_config_dir

config_dir = user_config_dir("hub_to_lab")
os.makedirs(config_dir, exist_ok=True)

ENV_FILE = os.path.join(config_dir, ".env")
load_dotenv(ENV_FILE)

def update_env_variable(key: str, value: str):

    if not os.path.exists(ENV_FILE):
        with open(ENV_FILE, 'w') as f:
            pass
    set_key(ENV_FILE, key, value)
